@@ -1,9 +1,9 @@
 # -*- coding: utf-8 -*-
 
 """
-Copyright (C) 2009-2012 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+Copyright (C) 2009-2014 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
-kajongg is free software you can redistribute it and/or modify
+Kajongg is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation either version 2 of the License, or
 (at your option) any later version.
@@ -20,21 +20,19 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 import datetime
 
-from kde import KIcon, KDialogButtonBox
-from PyQt4.QtCore import Qt, QVariant, \
-        QAbstractTableModel
-from PyQt4.QtGui import QDialog, QDialogButtonBox, QWidget, \
+from qt import Qt, QVariant, QAbstractTableModel
+from qt import QDialog, QDialogButtonBox, QWidget, \
         QHBoxLayout, QVBoxLayout, QAbstractItemView, \
         QItemSelectionModel, QGridLayout, QColor, QPalette
 
-from kde import KApplication
+from kde import KApplication, KIcon, KDialogButtonBox
 
 from genericdelegates import RichTextColumnDelegate
 
-from util import m18n, m18nc, m18nE, logDebug
+from log import m18n, m18nc, m18nE, logDebug
 from statesaver import StateSaver
 from rule import Ruleset
-from guiutil import ListComboBox, MJTableView
+from guiutil import ListComboBox, MJTableView, decorateWindow
 from differ import RulesetDiffer
 from common import Internal, Debug
 from modeltest import ModelTest
@@ -42,7 +40,7 @@ from chat import ChatMessage, ChatWindow
 
 class TablesModel(QAbstractTableModel):
     """a model for our tables"""
-    def __init__(self, tables, parent = None):
+    def __init__(self, tables, parent=None):
         super(TablesModel, self).__init__(parent)
         self.tables = tables
         assert isinstance(tables, list)
@@ -129,7 +127,7 @@ class SelectRuleset(QDialog):
     """a dialog for selecting a ruleset"""
     def __init__(self, server=None):
         QDialog.__init__(self, None)
-        self.setWindowTitle(m18n('Select a ruleset') + ' - Kajongg')
+        decorateWindow(self, m18n('Select a ruleset'))
         self.buttonBox = KDialogButtonBox(self)
         self.buttonBox.setStandardButtons(QDialogButtonBox.Cancel|QDialogButtonBox.Ok)
         self.buttonBox.accepted.connect(self.accept)
@@ -171,7 +169,7 @@ class TableList(QWidget):
         self.leaveButton.clicked.connect(self.leaveTable)
         self.leaveButton.setIcon(KIcon("list-remove-user"))
         self.leaveButton.setToolTip(m18n("Leave a table"))
-        self.compareButton = buttonBox.addButton(m18nc('Kajongg-Ruleset','Compare'), QDialogButtonBox.AcceptRole)
+        self.compareButton = buttonBox.addButton(m18nc('Kajongg-Ruleset', 'Compare'), QDialogButtonBox.AcceptRole)
         self.compareButton.clicked.connect(self.compareRuleset)
         self.compareButton.setIcon(KIcon("preferences-plugin-script"))
         self.compareButton.setToolTip(m18n('Compare the rules of this table with my own rulesets'))
@@ -197,17 +195,19 @@ class TableList(QWidget):
 
     def hideEvent(self, dummyEvent): # pylint: disable=no-self-use
         """table window hides"""
-        field = Internal.field
-        field.startingGame = False
+        scene = Internal.scene
+        if scene:
+            scene.startingGame = False
         model = self.view.model()
         if model:
             for table in model.tables:
                 if table.chatWindow:
                     table.chatWindow.hide()
                     table.chatWindow = None
-        if not field.game or field.game.client != self.client:
-            # do we still need this connection?
-            self.client.logout()
+        if scene:
+            if not scene.game or scene.game.client != self.client:
+                # do we still need this connection?
+                self.client.logout()
 
     def chat(self):
         """chat. Only generate ChatWindow after the
@@ -216,7 +216,7 @@ class TableList(QWidget):
         def initChat(_):
             """now that we were able to send the message to the server
             instantiate the chat window"""
-            table.chatWindow = ChatWindow(table)
+            table.chatWindow = ChatWindow(table=table)
             table.chatWindow.receiveLine(msg)
         table = self.selectedTable()
         if not table.chatWindow:
@@ -234,7 +234,7 @@ class TableList(QWidget):
             title = m18n('Local Games with Ruleset %1', self.client.ruleset.name)
         else:
             title = m18n('Tables at %1', self.client.connection.url)
-        self.setWindowTitle(' - '.join([self.client.name, title, 'Kajongg']))
+        decorateWindow(self, ' - '.join([self.client.name, title]))
         self.view.hideColumn(1)
         tableCount = self.view.model().rowCount(None) if self.view.model() else 0
         self.view.showColumn(0)
@@ -262,8 +262,8 @@ class TableList(QWidget):
             not running and
             not table.isOnline(self.client.name) and
             (self.client.name in table.playerNames) == suspended)
-        self.leaveButton.setVisible(not (suspendedLocalGame))
-        self.compareButton.setVisible(not (suspendedLocalGame))
+        self.leaveButton.setVisible(not suspendedLocalGame)
+        self.compareButton.setVisible(not suspendedLocalGame)
         self.startButton.setVisible(not suspended)
         if suspendedLocalGame:
             self.newButton.setToolTip(m18n("Start a new game"))
@@ -354,7 +354,7 @@ class TableList(QWidget):
         tables that also exist locally. In theory all suspended games should
         exist locally but there might have been bugs or somebody might
         have removed the local database like when reinstalling linux"""
-        if not Internal.field:
+        if not Internal.scene:
             return
         if Debug.table:
             for table in tables:
